#include "fire_interaction_viewer.h"
#include "box.h"
#include "box_interact.h"
#include "seqlogo.h"
#include "seqlogo_vertical.h"
#include "parser.h"
#include "scalebar.h"
#include "box_text.h"

#include <QtGui>
#include <QGraphicsSvgItem>
#include <QDebug>


#include <iostream>
#include <string>
#include <vector>
#include <sstream>
#include <algorithm>
#include <iterator>
#include <math.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fstream>
#include <map>
#include <algorithm>

using namespace std;

fire_interaction_viewer::fire_interaction_viewer(string filename)
{
    fire_results results = parse_fire(filename, "DNA_RNA");

    if (results.interaction.status == 1 && results.general.motifs.size() > 0) {
    } else {
        exit(1);
    }

    int motifcount = results.general.motifs.size();
    int bincount = results.general.bincount;
    int clusters = results.general.clusters;

    map <int, int> order;
    int counter = 0;
    // order motifs by mi and cluster
    vector<nucleotide_motif> ordered_candidates;
    for (int i = 0; i < clusters; i++) {
        for (int j = 0; j < motifcount; j++) {
            if (results.interaction.motifs[j].cluster == i) {
                ordered_candidates.push_back(results.interaction.motifs[j]);
                order.insert(make_pair(counter, j));
                counter++;
            }
        }
    }


        this->setWindowTitle(tr("FIRE: ") + QString::fromStdString(filename));
    this->setWindowIcon(QIcon(":/icon/icon"));

    // set up stage for drawing
    this->setViewportUpdateMode(FullViewportUpdate);
    this->setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);

    // set up pixel values
    int top_margin = 8;
    int bottom_margin = 8;
    int left_margin = 8;
    int right_margin = 8;

    int y_colorbar_offset = 30;
    int y_offset = 0;

    int horiz_gap = 12;

    int y_header_offset = 100 + 2*horiz_gap;
//    int bin_height = 50;

    int x_offset = 0;
//    int x_labelwidth = 25;

//    int bin_width = 8;
//    int cluster_gap = 4;


    int scalebar_width = 550;

    int box_count = 10;
    int box_width = 40;

    // determine scene height, width
    int height = top_margin + y_colorbar_offset + y_offset + y_header_offset + motifcount *(box_width+1) + bottom_margin;
    int width = left_margin + x_offset +  motifcount *(box_width+1) + 50 + 80 + right_margin;
    if (width < left_margin + scalebar_width + right_margin) {
        width = left_margin + scalebar_width + right_margin;
    }

    // define scene
    QRectF bounds (0, 0, width, height);
    scene = new QGraphicsScene(bounds, this);

    // define background color; currently light grey
    QColor background = QColor(230, 230, 230);

    // add background
    box *base = new box (width, height, background, background, 0, 0, false);
    scene->addItem(base);

    scene->setItemIndexMethod(QGraphicsScene::NoIndex);
    this->setScene(scene);

    // scalebar section
    QGraphicsItem *scale = new scalebar(scalebar_width, y_colorbar_offset, left_margin, top_margin,-0.01,0,0.01,":/heatmap/hot", "negative", "positive");
    scene->addItem(scale);
    // end scalebar section

    int left_x = left_margin+x_offset;
    int top_y = top_margin+y_colorbar_offset+y_offset;

    QGraphicsItem **vertical_locations = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        vertical_locations[i] = new box_text(box_width, 40, background, background, left_x + i*(box_width+1), top_y, QString::fromStdString(ordered_candidates[i].location), true, false, true);

        if (QString::fromStdString(ordered_candidates[i].location) == "5'") {
            vertical_locations[i]->setToolTip("DNA motif candidate");
        } else {
            vertical_locations[i]->setToolTip("RNA motif candidate");
        }

        scene->addItem(vertical_locations[i]);
    }

    QGraphicsItem **vertical_candidates = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        QString candidate = QString::fromStdString(ordered_candidates[i].candidate);
        if (QString::fromStdString(ordered_candidates[i].location) == "5'") {
        } else {
            candidate = candidate.replace("T", "U");
        }
        vertical_candidates[i] = new seqlogo_vertical(box_width, 60, left_x + i*(box_width+1), top_y+horiz_gap+40, candidate, background);
        vertical_candidates[i]->setToolTip(candidate);
        scene->addItem(vertical_candidates[i]);
    }

    // color determination adapted from iPAGE GUI (goodarzi)
    QList<QColor> colormap;
    double cmax = 0.01;
    double cmin = -0.01;
    double cmid = 0;
    QFile cmapfile(":/heatmap/hot");
    cmapfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QString cmapline = cmapfile.readLine() ;
    while (cmapline.length()>1){
        cmapline = cmapline.replace(QRegExp("\\s$"), QString::null);
        QStringList cmaplist = cmapline.split(" ");
        colormap.append(QColor(cmaplist[0].toFloat()*255, cmaplist[1].toFloat()*255, cmaplist[2].toFloat()*255));
        cmapline = cmapfile.readLine() ;
    }

    int color_count = colormap.length()/2 ;
    // end color determination adapted from iPAGE GUI (goodarzi)

    QGraphicsItem ***graphicbins = new QGraphicsItem **[motifcount];
    for (int i = 0; i < motifcount; i++) {
        graphicbins[i] = new QGraphicsItem *[motifcount];
        for (int j = 0; j < motifcount; j++) {
            int matrix_i = order[i];
            int matrix_j = order[j];

            double v = 0;

            if (QString::fromStdString(ordered_candidates[i].candidate) != QString::fromStdString(ordered_candidates[j].candidate)) {
                if(fabs(results.interaction.interaction_matrix_r3[matrix_i][matrix_j]) < pow(10,-10)) {
                    v = 0;
                } else {
                    v = (results.interaction.interaction_matrix_r0[matrix_i][matrix_j] * results.interaction.interaction_matrix_r3[matrix_i][matrix_j])/fabs(results.interaction.interaction_matrix_r3[matrix_i][matrix_j]);
                }
            } else {
                v = 1;
            }

            QColor binfill;
            // adapted from iPAGE GUI (goodarzi)
            if (v > 0){
                int r = (v-cmid)/(cmax-cmid)*color_count + color_count ;
                if (r>=colormap.length()) {
                    r = colormap.length()-1 ;
                }
                binfill = colormap[r];
            }else{
                int r = (1-(v-cmid)/(cmin-cmid))*color_count ;
                if (r<0) {
                    r=0 ;
                }
                binfill = colormap[r];
            }
            // end adapted from iPAGE GUI

            QColor lborder = binfill;
            QColor rborder = binfill;
            QColor tborder = binfill;
            QColor bborder = binfill;
            bool plus = false;

            if (j != motifcount -1 && ordered_candidates[j].cluster != ordered_candidates[j+1].cluster) {
                rborder = QColor(0,0,0);
            }

            if (i != motifcount -1 && ordered_candidates[i].cluster != ordered_candidates[i+1].cluster) {
                bborder = QColor(0,0,0);
            }

            if (j > 0 && ordered_candidates[j].cluster != ordered_candidates[j-1].cluster) {
               lborder = QColor(0,0,0);
            }

            if (i > 0 && ordered_candidates[i].cluster != ordered_candidates[i-1].cluster) {
                tborder = QColor(0,0,0);
            }

            if (!isnan(results.interaction.interaction_matrix_r1[matrix_i][matrix_j]) && results.interaction.interaction_matrix_r1[matrix_i][matrix_j] == 0) {
                if (QString::fromStdString(ordered_candidates[i].location) != QString::fromStdString(ordered_candidates[j].location)) {
                    lborder = QColor(93,252,10);
                    rborder = QColor(93,252,10);
                    tborder = QColor(93,252,10);
                    bborder = QColor(93,252,10);
                } else {
                    if (QString::fromStdString(ordered_candidates[i].location) == "5'") {
                        lborder = QColor(0,0,255);
                        rborder = QColor(0,0,255);
                        tborder = QColor(0,0,255);
                        bborder = QColor(0,0,255);
                    } else {
                        lborder = QColor(255,102,255);
                        rborder = QColor(255,102,255);
                        tborder = QColor(255,102,255);
                        bborder = QColor(255,102,255);
                    }
                }
                if (!isnan(results.interaction.interaction_matrix_r4[matrix_i][matrix_j]) && results.interaction.interaction_matrix_r4[matrix_i][matrix_j] < 100 && results.interaction.interaction_matrix_r4[matrix_i][matrix_j] > 0) {
                    plus = true;
                }
            }

            if (i == 0) {
               tborder = QColor(0,0,0);
            }

            if (i == motifcount -1) {
                bborder = QColor(0,0,0);
            }

            if (j == 0) {
                lborder = QColor(0,0,0);
            }

            if (j == motifcount -1) {
                rborder = QColor(0,0,0);
            }

            graphicbins[i][j] = new box_interact(box_width, box_width, binfill, binfill, left_x + j*(box_width+1), top_y+y_header_offset+i*(box_width+1),lborder, rborder, tborder, bborder, plus, false);
            if (v!=0) {
                graphicbins[i][j]->setToolTip(QString::number(v));
            }
            scene->addItem(graphicbins[i][j]);
        }
    }

    QGraphicsItem **horizontal_locations = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        horizontal_locations[i] = new box_text(box_width, 40, background, background, left_x + motifcount*(box_width+1) + horiz_gap+60+horiz_gap, top_y+y_header_offset+i*(box_width+1), QString::fromStdString(ordered_candidates[i].location));

        if (QString::fromStdString(ordered_candidates[i].location) == "5'") {
            horizontal_locations[i]->setToolTip("DNA motif candidate");
        } else {
            horizontal_locations[i]->setToolTip("RNA motif candidate");
        }

        scene->addItem(horizontal_locations[i]);
    }

    QGraphicsItem **horizontal_candidates = new QGraphicsItem *[motifcount];
    for (int i = 0; i < motifcount; i++) {
        QString candidate = QString::fromStdString(ordered_candidates[i].candidate);
        if (QString::fromStdString(ordered_candidates[i].location) == "5'") {
        } else {
            candidate = candidate.replace("T", "U");
        }
        horizontal_candidates[i] = new seqlogo(60, box_width, left_x + motifcount*(box_width+1) + horiz_gap, top_y+y_header_offset+i*(box_width+1), candidate, background);
        horizontal_candidates[i]->setToolTip(candidate);
        scene->addItem(horizontal_candidates[i]);
    }


}
