#include "scalebar.h"
#include <QtGui>

scalebar::scalebar(int width, int height, int xoffset, int yoffset, double cmin, double cmid, double cmax, QString colorfile, QString min, QString max)
{
    this->width = width;
    this->height = height;
    this->xoffset = xoffset;
    this->yoffset = yoffset;
    this->cmin = cmin;
    this->cmid = cmid;
    this->cmax = cmax;
    this->colorfile = colorfile;
    this->min = min;
    this->max = max;
}

QRectF scalebar::boundingRect() const
{
    return QRectF(this->xoffset, this->yoffset, this->width, this->height);
}

QPainterPath scalebar::shape() const
{
    QPainterPath path;
    path.addRect(this->xoffset, this->yoffset, this->width, this->height);
    return path;
}

void scalebar::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
    // scale drawing adapted from iPAGE GUI (goodarzi)
    int res = 50;
    double w_s = 2;
    double h_s = this->height*3/4;

    // color map parsing adapted from iPAGE GUI (goodarzi)
    QList<QColor> colormap;
    QFile cmapfile(colorfile);
    cmapfile.open(QIODevice::ReadOnly | QIODevice::Text);
    QString cmapline = cmapfile.readLine() ;
    while (cmapline.length()>1){
        cmapline = cmapline.replace(QRegExp("\\s$"), QString::null);
        QStringList cmaplist = cmapline.split(" ");
        colormap.append(QColor(cmaplist[0].toFloat()*255, cmaplist[1].toFloat()*255, cmaplist[2].toFloat()*255));
        cmapline = cmapfile.readLine() ;
    }

    int color_count = colormap.length()/2 ;

    // end color map parsing adapted from iPAGE GUI (goodarzi)

    double labeloffset = (this->width - 2*res*w_s)/9;

    for(int i=0 ; i<res ; i++){
        int idx = ((res-i) * color_count / res) +  color_count ;
        if (idx >= colormap.length())
            idx = colormap.length()-1 ;
        painter->setBrush(colormap[idx]) ;
        painter->setPen(colormap[idx]);
        painter->drawRect(this->xoffset + labeloffset*4 + i*w_s, this->yoffset, w_s, h_s) ;
    }

    for(int i=0 ; i<res ; i++){
        int idx = ((res-i) * color_count / res) ;
        if (idx <0)
            idx = 0 ;
        painter->setBrush(colormap[idx]) ;
        painter->setPen(colormap[idx]);
        painter->drawRect(this->xoffset + labeloffset*5+res*w_s+i*w_s, this->yoffset, w_s, h_s) ;
    }

    painter->setPen(QColor(0,0,0));
    painter->setFont(QFont("Helvetica", this->height/4));
    painter->drawText(QRect(this->xoffset, this->yoffset, labeloffset*3, h_s),Qt::AlignVCenter|Qt::AlignLeft, this->max);
    painter->drawText(QRect(this->xoffset+labeloffset*3, this->yoffset, labeloffset, h_s),Qt::AlignCenter, QString::number(this->cmax));
    painter->drawText(QRect(this->xoffset+labeloffset*4+res*w_s, this->yoffset, labeloffset, h_s),Qt::AlignCenter, QString::number(this->cmid));
    painter->drawText(QRect(this->xoffset+labeloffset*5+2*res*w_s, this->yoffset, labeloffset, h_s),Qt::AlignCenter, QString::number(this->cmin));
    painter->drawText(QRect(this->xoffset+labeloffset*6+2*res*w_s, this->yoffset, labeloffset*3, h_s),Qt::AlignVCenter|Qt::AlignRight, this->min);

    // end scale drawing adapted from iPAGE GUI (goodarzi)
}

void scalebar::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
//    if(this->interact) {
//        event->accept();
//        this->width+=10;
//        //location to launch new info dialog
//        update();
//    }
    QGraphicsItem::mousePressEvent(event);
}
