#include "seqlogo.h"
#include "math.h"
#include "seqlogodialog.h"

#include <QtGui>
#include <QGraphicsSvgItem>

seqlogo::seqlogo(int width, int height, int xoffset, int yoffset, QString candidate, QColor background, bool interact)
{
    this->width = width;
    this->height = height;
    this->xoffset = xoffset;
    this->yoffset = yoffset;
    this->interact = interact;
    this->candidate = candidate;
    this->fill = background;
    this->border = background;
}

QRectF seqlogo::boundingRect() const
{
    return QRectF(this->xoffset, this->yoffset, this->width, this->height);
}

QPainterPath seqlogo::shape() const
{
    QPainterPath path;
    path.addRect(this->xoffset, this->yoffset, this->width, this->height);
    return path;
}

void seqlogo::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
    painter->setBrush(this->fill);
    painter->setPen(this->border);
    painter->drawRect(QRect(this->xoffset, this->yoffset, this->width, this->height));

    int length = this->candidate.length();

    int motifcharcounter = 0;
    for(int i = 0; i < length; i++) {
        if (candidate[i] == '[') {
            i++;
            while(candidate[i] != ']') {
                // concat string
                i++;
            }
            motifcharcounter++;
        } else if (candidate[i] == '.') {
            motifcharcounter++;
        } else {
            motifcharcounter++;
        }
    }

    int motiflength = motifcharcounter++;
    this->logo = new QGraphicsSvgItem *[motiflength];

    // reset motifcharcounter
    motifcharcounter = 0;

    QGraphicsSvgItem *sizetest = new QGraphicsSvgItem(":/svg/G");
    double sizew = sizetest->boundingRect().width();
    double sizeh = sizetest->boundingRect().height();

    double scalefactor;
    if (this->height/sizeh > this->width/(sizew*motiflength)) {
        scalefactor = this->width/(sizew*motiflength);
    } else {
        scalefactor = (this->height*3/4)/sizeh;
    }
    // vertical centering if scaling requires
    double buffer_top = (this->height-scalefactor*sizeh)/2;

    for(int i = 0; i < length; i++) {
        if (candidate[i] == '[') {
            i++;
            QString charfile = ":/svg/";
            while(candidate[i] != ']') {
                // concat string
                charfile += candidate[i];
                i++;
            }
            // dump
            this->logo[motifcharcounter] = new QGraphicsSvgItem(charfile, this);
            this->logo[motifcharcounter]->scale(scalefactor,scalefactor);
            this->logo[motifcharcounter]->setPos(this->xoffset+sizew*scalefactor*motifcharcounter, this->yoffset+buffer_top+scalefactor*(sizeh-this->logo[motifcharcounter]->boundingRect().height()));
            motifcharcounter++;
        } else if (candidate[i] == '.') {
            motifcharcounter++;
        } else {
            //single character
            QString charfile = ":/svg/" + candidate[i];
            this->logo[motifcharcounter] = new QGraphicsSvgItem(charfile, this);
            this->logo[motifcharcounter]->scale(scalefactor,scalefactor);
            this->logo[motifcharcounter]->setPos(this->xoffset+sizew*scalefactor*motifcharcounter, this->yoffset+buffer_top+scalefactor*(sizeh-this->logo[motifcharcounter]->boundingRect().height()));
            motifcharcounter++;
        }
    }
}

void seqlogo::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if(this->interact) {
        event->accept();
        seqlogodialog *dialog = new seqlogodialog(this->candidate);
        dialog->show();
        //location to launch new info dialog
        //update();
    }
    QGraphicsItem::mousePressEvent(event);
}
