#include "seqlogo_vertical.h"
#include "math.h"
#include "seqlogodialog.h"

#include <QtGui>
#include <QGraphicsSvgItem>
#include <QDebug>

seqlogo_vertical::seqlogo_vertical(int width, int height, int xoffset, int yoffset, QString candidate, QColor background, bool interact)
{
    this->width = width;
    this->height = height;
    this->xoffset = xoffset;
    this->yoffset = yoffset;
    this->interact = interact;
    this->candidate = candidate;
    this->fill = background;
    this->border = background;
}

QRectF seqlogo_vertical::boundingRect() const
{
    return QRectF(this->xoffset, this->yoffset, this->width, this->height);
}

QPainterPath seqlogo_vertical::shape() const
{
    QPainterPath path;
    path.addRect(this->xoffset, this->yoffset, this->width, this->height);
    return path;
}

void seqlogo_vertical::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget) {
    painter->setBrush(this->fill);
    painter->setPen(this->border);
    painter->drawRect(QRect(this->xoffset, this->yoffset, this->width, this->height));

    int length = this->candidate.length();

    int motifcharcounter = 0;
    for(int i = 0; i < length; i++) {
        if (candidate[i] == '[') {
            i++;
            while(candidate[i] != ']') {
                // concat string
                i++;
            }
            motifcharcounter++;
        } else if (candidate[i] == '.') {
            motifcharcounter++;
        } else {
            motifcharcounter++;
        }
    }

    int motiflength = motifcharcounter++;
    this->logo = new QGraphicsSvgItem *[motiflength];

    // reset motifcharcounter
    motifcharcounter = 0;

    QGraphicsSvgItem *sizetest = new QGraphicsSvgItem(":/svg/G");
    sizetest->rotate(-90);
    double sizeh = sizetest->boundingRect().width();
    double sizew = sizetest->boundingRect().height();

    double scalefactor;
    if (this->height/(sizeh*motiflength) > this->width/(sizew)) {
        scalefactor = (this->width*3/4)/(sizew);
    } else {
        scalefactor = (this->height)/(sizeh*motiflength);
    }
    // vertical centering if scaling requires
    double buffer_top = (this->width-scalefactor*sizew)/2;

    for(int i = 0; i < length; i++) {
        if (candidate[i] == '[') {
            i++;
            QString charfile = ":/svg/";
            while(candidate[i] != ']') {
                // concat string
                charfile += candidate[i];
                i++;
            }
            // dump
            this->logo[motifcharcounter] = new QGraphicsSvgItem(charfile, this);
            this->logo[motifcharcounter]->scale(scalefactor,scalefactor);
            this->logo[motifcharcounter]->rotate(-90);
            this->logo[motifcharcounter]->setPos(this->xoffset+buffer_top+scalefactor*(sizew - this->logo[motifcharcounter]->boundingRect().height()), this->yoffset+this->height-sizeh*scalefactor*motifcharcounter);
//            this->logo[motifcharcounter]->setPos(this->xoffset+sizew*scalefactor*motifcharcounter, this->yoffset+buffer_top+scalefactor*(sizeh-this->logo[motifcharcounter]->boundingRect().height()));
            motifcharcounter++;
        } else if (candidate[i] == '.') {
            motifcharcounter++;
        } else {
            //single character
            QString charfile = ":/svg/" + candidate[i];
            this->logo[motifcharcounter] = new QGraphicsSvgItem(charfile, this);
            this->logo[motifcharcounter]->scale(scalefactor,scalefactor);
            this->logo[motifcharcounter]->rotate(-90);
            this->logo[motifcharcounter]->setPos(this->xoffset+buffer_top+scalefactor*(sizew-this->logo[motifcharcounter]->boundingRect().height()), this->yoffset+this->height-sizeh*scalefactor*motifcharcounter);
//            this->logo[motifcharcounter]->setPos(this->xoffset+sizew*scalefactor*motifcharcounter, this->yoffset+buffer_top+scalefactor*(sizeh-this->logo[motifcharcounter]->boundingRect().height()));
            motifcharcounter++;
        }
    }
}

void seqlogo_vertical::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    if(this->interact) {
        event->accept();
        seqlogodialog *dialog = new seqlogodialog(this->candidate);
        dialog->show();
        //location to launch new info dialog
        //update();
    }
    QGraphicsItem::mousePressEvent(event);
}
