#include <QtGui>
#include <QDebug>

#include <string>

#include "welcome.h"
#include "ui_welcome.h"
#include "parser.h"
#include "resultchoice.h"

#include "fire_viewer.h"
#include "fire_interaction_viewer.h"
#include "firepro_viewer.h"
#include "firepro_interaction_viewer.h"
#include "ipage_viewer.h"
#include "prmg_viewer.h"
#include "prmg_protein_viewer.h"

using namespace std;

welcome::welcome(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::welcome)
{
    ui->setupUi(this);

    connect(ui->loadButton, SIGNAL(clicked()), this, SLOT(process_loadButton()));
    connect(ui->resetButton, SIGNAL(clicked()), this, SLOT(process_resetButton()));
    connect(ui->expfileEdit, SIGNAL(selectionChanged()), this, SLOT(process_chooseExpfile()));
    this->setWindowTitle(tr("IGET Viewer"));
    this->setWindowIcon(QIcon(":/icon/icon"));
 }

void welcome::process_chooseExpfile() {
    if(ui->expfileEdit->text().length() == 0) {
        QString fileName = QFileDialog::getOpenFileName(this, tr("Choose information file"), "", tr("information files (*.txt *.* *)"));
        ui->expfileEdit->setText(fileName);
    }
}

void welcome::process_loadButton () {
    int haveexpfile = 0;

    if(ui->expfileEdit->text().length() == 0) {
        QString fileName = QFileDialog::getOpenFileName(this, tr("Choose information file"), "", tr("information files (*.txt *.* *)"));
        ui->expfileEdit->setText(fileName);
        if(ui->expfileEdit->text().length() > 0) {
            haveexpfile++;
        }
    } else {
        haveexpfile++;
    }

    if (haveexpfile == 0) {
        QMessageBox::StandardButton reply;
        reply = QMessageBox::warning(this, tr("IGET Viewer"), tr("No information file was specified. Please specify an information file and try again."), QMessageBox::Ok);
    } else {
        string filename = ui->expfileEdit->text().toStdString();
        string presence = determine_results(filename);

        int resultcount = 0;
        for (int i = 0; i < presence.length(); i++) {
            if (presence.at(i) == '1') {
                if (i < 3 && resultcount > 0) {
                } else {
                    resultcount++;
                }
            }
        }

        if (resultcount > 1){

            resultchoice *rcdialog = new resultchoice(filename, presence, this);
            rcdialog->show();

        } else if (resultcount == 1) {

            if (presence.at(2) == '1') {
                fire_viewer *fire_viewer_dialog = new fire_viewer(filename);
                fire_viewer_dialog->show();
                fire_interaction_viewer *fire_interaction_viewer_dialog = new fire_interaction_viewer(filename);
                fire_interaction_viewer_dialog->show();
            } else if (presence.at(3) == '1') {
                firepro_viewer *firepro_viewer_dialog = new firepro_viewer(filename);
                firepro_viewer_dialog->show();
                firepro_interaction_viewer *firepro_interaction_viewer_dialog = new firepro_interaction_viewer(filename);
                firepro_interaction_viewer_dialog->show();
            } else if (presence.at(4) == '1') {
                ipage_viewer *ipage_viewer_dialog = new ipage_viewer(filename);
                ipage_viewer_dialog->show();
            } else if (presence.at(5) == '1') {
                prmg_viewer *prmg_viewer_dialog = new prmg_viewer(filename);
                prmg_viewer_dialog->show();
            } else if (presence.at(6) == '1') {
                prmg_protein_viewer *prmg_protein_viewer_dialog = new prmg_protein_viewer(filename);
                prmg_protein_viewer_dialog->show();
            }

        } else {
            QMessageBox::warning(this, tr("IGET Viewer"), tr("No results were found. Please specify another information file and try again."), QMessageBox::Ok);
        }
    }
}

void welcome::process_resetButton () {
    ui->expfileEdit->clear();
}
